#!/usr/bin/env python3
##
## -----------------------------------------------------------------
##    This file is part of WAPT Software Deployment
##    Copyright (C) 2012 - 2022  Tranquil IT https://www.tranquil.it
##    All Rights Reserved.
##
##    WAPT helps systems administrators to efficiently deploy
##    setup, update and configure applications.
## ------------------------------------------------------------------
##

import os
import sys
import platform
import time
import logging
import traceback
import json
import datetime

from flask import request, Flask, Response, send_from_directory, session, redirect, url_for, abort, render_template, flash
from itsdangerous import TimedJSONWebSignatureSerializer

from waptserver.app import app

from waptserver.config import __version__
from waptserver.model import Hosts
from waptutils import jsondump, ensure_list, ensure_unicode
from waptserver.utils import EWaptMissingParameter, EWaptSignalReceived, EWaptTimeoutWaitingForResult, EWaptUnknownHost

from waptcrypto import SSLCertificate, SSLVerifyException
from licencing import WaptActivatedLicence,EWaptBadlicence
import waptlicences

from waptserver.utils import wapt_root_dir


logger = logging.getLogger()

__all__ = [
    'make_response',
    'make_response_from_exception',
    'get_server_uuid',
    'get_secured_token_generator',
]

# API V2 #####


def make_response(result={}, success=True, error_code='', msg='', status=200, request_time=None, headers = None):
    data = dict(
        success=success,
    )

    if msg:
        data['msg']=msg
    if not success:
        data['error_code'] = error_code
    else:
        data['result'] = result
    if request_time:
        data['request_time'] = request_time
    return Response(
        response=jsondump(data),
        status=status,
        mimetype='application/json',
        headers = headers)


def make_response_from_exception(exception, error_code='', status=200):
    """Return a error flask http response from an exception object
        success : False
        msg : message from exception
        error_code : classname of exception if not provided
        status: 200 if not provided
    """
    if not error_code:
        error_code = type(exception).__name__.lower()
    data = dict(
        success=False,
        error_code=error_code
    )

    logger.info(traceback.format_exc())

    data['msg'] = 'Error on server:\n%s' % (repr(exception))
    return Response(
        response=jsondump(data),
        status=status,
        mimetype='application/json')


def get_server_uuid():
    """Returns this server UUID as configured in configuration file waptserver.ini
    """
    return app.conf.get('server_uuid', None)

def get_secured_token_generator(secret_key=None):
    if not secret_key:
        secret_key = app.conf['secret_key']
    return TimedJSONWebSignatureSerializer(secret_key, expires_in=app.conf['token_lifetime'])


class EWaptLicenceDirectorySSLCertificateNotExist(Exception):
    pass

class LicenceStatus:
    lsUnknown=0
    lsValid=1
    lsNotJson=2
    lsBadServer=3
    lsBadSignature=4
    lsNotYetValid=5
    lsExpired=6
    lsDuplicated=7

    text = {
        0:'Unknown',
        1:'Valid',
        2:'Not Json',
        3:'Bad Server',
        4:'Bad Signature',
        5:'Not Yet Valid',
        6:'Expired',
        7:'Duplicated'
    }

class WaptLicencesList():

    def __init__(self, conf=None):
        self._conf = conf
        self._json_path = None
        self._ssl_certificate = None

        self.hosts_count = 0
        self.licences_list = None
        self.timestamp_json = None

    @property
    def conf(self):
        return self._conf

    @conf.setter
    def conf(self,value):
        self._conf = value
        self._json_path = None
        self._ssl_certificate = None

        self.hosts_count = 0
        self.licences_list = None
        self.timestamp_json = None

    @property
    def json_path(self):
        if not self._json_path:
            self._json_path = os.path.join(os.path.dirname(self.conf['wapt_folder']), 'licences.json')
        return self._json_path

    def is_enterprise(self):
        self.load_licences()
        return waptlicences.is_enterprise()

    def check_licence_count(self,licences_data):
        data_check = waptlicences.check_valid_licences_count(licences_data,self.conf.get('server_uuid', ''),0)
        self.hosts_count = data_check['count']
        self.licenced_to = data_check['licenced_to']
        self.licences_list = data_check['licences']
        return self.hosts_count

    def check_licence(self, licence_data):
        """Check that a licence is properly signed.

        If the licence has a server_uuid, certificate used to check is
        the relicencing cert
        Else it is the TIS licencing cert

        Args:
            licence_data (str): json of signed licence object

        Returns: int
            0:'Valid',
            1:'Unknown',
            2:'Not Json',
            3:'Bad Server',
            4:'Bad Signature',
            5:'Not Yet Valid',
            6:'Expired',
            7:'Duplicated'
        """
        if not isinstance(licence_data,str):
            licence_data = jsondump(licence_data)
        return waptlicences.check_licence_signature(licence_data,get_server_uuid())

    def check_time_validity_licence(self, licence_data):
        """Check that a licence is not expired.
        """
        return WaptActivatedLicence(licence_data_dict=licence_data).is_valid_until_from()

    def add_licence(self, licence = None):
        self.load_licences()
        if isinstance(licence, (str, bytes)):
            new_licence = json.loads(licence)
        else:
            new_licence = licence
        if new_licence['licence_nr'] in [l['licence_nr'] for l in self.licences_list]:
            return (None,LicenceStatus.lsDuplicated,'Licence %s already exists on server' % new_licence['licence_nr'])
        status = self.check_licence(new_licence)
        if status == LicenceStatus.lsValid:
            self.licences_list.append(new_licence)
            self.write_licences()
            return (new_licence, status, 'Activation of %(licenced_to)s-%(licence_nr)s OK' % new_licence)
        else:
            return (None, status, LicenceStatus.text[status])

    def load_licences(self):
        if self.timestamp_json is None or not os.path.isfile(self.json_path)\
                or self.timestamp_json < os.path.getmtime(self.json_path)\
                or self.licences_list is None:
            try:
                with open(self.json_path, 'r') as json_licences:
                    licences_data = json_licences.read()
            except (FileNotFoundError, json.decoder.JSONDecodeError):
                self.licences_list = []
                self.write_licences()
                licences_data = '[]'
            self.check_licence_count(licences_data)
            self.timestamp_json = os.path.getmtime(self.json_path)

    def write_licences(self):
        with open(self.json_path, 'w') as json_licences:
            json_licences.write(json.dumps(self.licences_list))
        # be sure to refresh valid licences
        self.load_licences()

    def remove_licences(self, licence_nr_list):
        self.load_licences()
        licence_to_remove = []
        for licence_nr in licence_nr_list:
            licence = self.get_licence(licence_nr)
            if licence is not None:
                licence_to_remove.append(licence)
                licence = None

        self.licences_list=[licence for licence in self.licences_list if licence not in licence_to_remove]
        self.write_licences()
        return licence_to_remove

    def remove_licence(self, licence_nr = None):
        if licence_nr is None:
            return None
        else:
            return self.remove_licences([licence_nr])[0]

    def get_licences(self):
        self.load_licences()
        return self.licences_list

    def get_licence(self, licence_nr = None):
        self.load_licences()
        for licence in self.licences_list:
            if licence['licence_nr'] == licence_nr:
                return licence
        return None

licences_list = WaptLicencesList()
